package org.dbxml.client.xmldb.services;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: DatabaseInstanceManager.java,v 1.10 2001/06/07 02:08:15 tom Exp $
 */

import org.dbxml.core.FaultCodes;
import org.dbxml.core.corba.EncodedBufferConverter;
import org.dbxml.core.corba.db.*;
import org.dbxml.client.xmldb.*;

import java.util.*;

import org.w3c.dom.Document;

import org.xmldb.api.base.*;
import org.xmldb.api.base.Collection;

/**
  * DatabaseInstanceManager enables management of the Database intance on the
  * server. It also enables you to shutdown the running server.<p>
  *
  * In order to get a reference to the DatabaseInstanceManager object you must have
  * administrative access to the server.
  */
public class DatabaseInstanceManager extends CommonConfigurable
      implements ManagedObject, Service {
   protected org.dbxml.core.corba.db.Database db = null;
   protected DatabaseManager manager;
   protected DatabaseImpl database = null;

   /**
    * Creates a new DatabaseInstanceManager instance
    */
   public DatabaseInstanceManager(DatabaseManager manager,
         org.dbxml.core.corba.db.Database db, DatabaseImpl database) {
      this.manager = manager;
      this.db = db;
      this.database = database;
   }

   public String getName() {
      return "DatabaseInstanceManager";
   }

   public String getVersion() {
      return "1.0";
   }

   /**
    * Not used for the DatabaseInstanceManager service.
    */
   public void setCollection(org.xmldb.api.base.Collection col)
         throws XMLDBException {
   }

   /**
    * Returns the name of this database
    *
    * @return The name of this database
    * @exception XMLDBException
    */
   public String getDatabaseName() throws XMLDBException  {
      try {
         return manager.getName();
      }
      catch (Exception e) {
         throw FaultCodes.createXMLDBException(e);
      }
   }

   /**
    * Shutsdown the Database instance
    *
    * @return the result of the shutdown operation
    * @exception XMLDBException
    */
   public void shutdown() throws XMLDBException {
      try {
         manager.shutdown();
         //return true;
      }
      catch (Exception e) {
         throw FaultCodes.createXMLDBException(e);
      }
   }

   /**
    * Removes the server side Servant for this object. Should be called when the
    * the object is no longer needed.
    */
   public void remove() {
      manager.remove();
   }
}

