package org.dbxml.tools.command;

/*
 * dbXML License, Version 1.0
 *
 *
 * Copyright (c) 1999-2001 The dbXML Group, L.L.C.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        The dbXML Group (http://www.dbxml.com/)."
 *    Alternately, this acknowledgment may appear in the software
 *    itself, if and wherever such third-party acknowledgments normally
 *    appear.
 *
 * 4. The names "dbXML" and "The dbXML Group" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@dbxml.com.
 *
 * 5. Products derived from this software may not be called "dbXML",
 *    nor may "dbXML" appear in their name, without prior written
 *    permission of The dbXML Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE DBXML GROUP OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: AddCollection.java,v 1.57 2001/09/04 11:13:48 kstaken Exp $
 */
import java.util.Hashtable;

import org.xmldb.api.*;
import org.dbxml.client.xmldb.services.*;
import org.dbxml.client.xmldb.*;

import org.xmldb.api.base.*;
import org.xmldb.api.DatabaseManager;

import org.dbxml.xml.dom.*;

import org.w3c.dom.*;

/**
 * AddCollection.java is designed to let the user create
 * new Collections and Nested Collections within the Database.
 */

public class AddCollection extends Command {

   /**
    * TODO: Description of the Method
    *
    * @param table Description of Parameter
    * @return Description of the Returned Value
    * @exception Exception Description of Exception
    */
   public boolean execute(Hashtable table) throws Exception {
      Collection col = null;
      Collection tempcol = null;

      try {

         // Verify that that collection passed in is not null
         if (table.get("collection") != null && table.get("nameOf") != null) {

            // Get a Collection reference to pass on to individual commands
            String colstring = normalizeCollectionURI((String) table.get("collection"));

            col = DatabaseManager.getCollection(colstring);

            if (col == null) {
               System.out.println("ERROR : Collection not found!");
               return false;
            }

            String newcol = (String) table.get("nameOf");

            // Create a new document for the collection passed in
            Document document = newCollection(newcol);

            // Create an instance of the collection manager and create the collection
            try {
               CollectionManager colman = (CollectionManager) col.getService("CollectionManager", XMLDBAPIVERSION);
               tempcol = colman.createCollection(newcol, document);
            }
            catch (Exception e) {
               System.out.println("ERROR : " + e.getMessage());
               return false;
            }

            System.out.println("Created : " + table.get("collection") + "/" + newcol);
         }
         else {
            System.out.println("ERROR : Collection Context and New Collection name required");
         }
      }

      catch (Exception e) {
         System.out.println("ERROR : " + e.getMessage());
         if (table.get("verbose").equals("true")) {
            org.dbxml.Debug.printStackTrace(e);
         }
         return false;
      }
      finally {
         // Release the collection objects
         if (col != null) {
            col.close();
         }
         if (tempcol != null) {
            tempcol.close();
         }
      }

      return true;
   }

   // Create a new document based on a Collection string
   /**
    * TODO: Description of the Method
    *
    * @param name Description of Parameter
    * @return Description of the Returned Value
    */
   public Document newCollection(String name) {
      String colName = "";
      String parName = name;

      int idx = parName.lastIndexOf("/");

      if (idx != -1) {
         colName = parName.substring(idx + 1);
         parName = parName.substring(0, idx);
      }
      else if (idx == -1) {
         colName = parName;
      }

      if (colName.equals("")) {
         System.out.println("Cannot create a NULL collection");
         return null;
      }

      Document doc = new DocumentImpl();

      Element colEle = doc.createElement("collection");
      colEle.setAttribute("compressed", "true");
      colEle.setAttribute("name", colName);

      doc.appendChild(colEle);

      Element filEle = doc.createElement("filer");
      filEle.setAttribute("class", "org.dbxml.core.filer.BTreeFiler");
      filEle.setAttribute("gzip", "true");

      colEle.appendChild(filEle);

      return doc;
   }

}



